/***************************************************
//Web: http://www.buydisplay.com
EastRising Technology Co.,LTD
Examples for ER-TFTM080-2 with Capacitive Touch Panel 
Display is Hardward SPI 4-Wire SPI Interface and 5V Power Supply
Capacitive Touch Panel is I2C Interface
Tested and worked with:
Works with Arduino 1.6.0 IDE  
Works with Arduino DUE,Arduino mega2560,Arduino UNO Board
Note: If you use our company's adapter board, you must remove the R6 resistor on the adapter board, otherwise the capacitive touch screen will not work.
****************************************************/
#include <stdint.h>
#include <SPI.h>
#include <Wire.h>

uint8_t addr  = 0x5d;  //CTP IIC ADDRESS

#include "Adafruit_GFX.h"
#include "Adafruit_RA8875.h"

//LCD:hardware SPI  CTP:hardware IIC
//Arduino DUE,Arduino mega2560,Arduino UNO
#define RA8875_INT 4
#define RA8875_CS 10 
#define RA8875_RESET 9
#define gt911_INT   7     
Adafruit_RA8875 tft = Adafruit_RA8875(RA8875_CS, RA8875_RESET);


unsigned char  GTP_CFG_DATA[] =
{
0x5A,0x20,0x03,0xE0,0x01,0x05,0x0D,0x00,
0x01,0x08,0x28,0x08,0x50,0x32,0x03,0x05,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x88,0x29,0x0A,0x35,0x37,
0xD3,0x07,0x00,0x00,0x01,0x81,0x02,0x1D,
0x00,0x01,0x00,0x00,0x00,0x03,0x64,0x32,
0x00,0x00,0x00,0x28,0x5A,0x94,0xC5,0x02,
0x00,0x00,0x00,0x00,0x98,0x2B,0x00,0x84,
0x33,0x00,0x74,0x3C,0x00,0x67,0x46,0x00,
0x5C,0x53,0x00,0x5C,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x02,0x04,0x06,0x08,0x0A,0x0C,0x0E,0x10,
0x12,0x14,0xFF,0xFF,0xFF,0xFF,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x02,
0x04,0x06,0x08,0x0F,0x10,0x12,0x16,0x18,
0x1C,0x1D,0x1E,0x1F,0x20,0x21,0x22,0xFF,
0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
0x25,0x01
};

struct TouchLocation
{
  uint16_t x;
  uint16_t y;
};
TouchLocation touchLocations[5];


void inttostr(uint16_t value,uint8_t *str);
uint8_t gt911_Send_Cfg(uint8_t * buf,uint16_t cfg_len);
void writegt911TouchRegister( uint16_t regAddr,uint8_t *val, uint16_t cnt);
uint8_t readgt911TouchAddr( uint16_t regAddr, uint8_t * pBuf, uint8_t len );
uint8_t readgt911TouchLocation( TouchLocation * pLoc, uint8_t num );
uint32_t dist(const TouchLocation & loc);
uint32_t dist(const TouchLocation & loc1, const TouchLocation & loc2);
bool sameLoc( const TouchLocation & loc, const TouchLocation & loc2 );

uint8_t buf[80];


uint8_t gt911_Send_Cfg(uint8_t * buf,uint16_t cfg_len)
{
	//uint8_t ret=0;
	uint8_t retry=0;
	for(retry=0;retry<5;retry++)
	{
		writegt911TouchRegister(0x8047,buf,cfg_len);
		//if(ret==0)break;
		delay(10);	 
	}
	//return ret;
}


void writegt911TouchRegister( uint16_t regAddr,uint8_t *val, uint16_t cnt)
{	uint16_t i=0;
  Wire.beginTransmission(addr);
   Wire.write( regAddr>>8 );  // register 0
  Wire.write( regAddr);  // register 0 
	for(i=0;i<cnt;i++,val++)//data
	{		
          Wire.write( *val );  // value
	}
  uint8_t retVal = Wire.endTransmission(); 
}



uint8_t readgt911TouchAddr( uint16_t regAddr, uint8_t * pBuf, uint8_t len )
{
  Wire.beginTransmission(addr);
  Wire.write( regAddr>>8 );  // register 0
  Wire.write( regAddr);  // register 0  
  uint8_t retVal = Wire.endTransmission();
  
  uint8_t returned = Wire.requestFrom(addr, len);    // request 1 bytes from slave device #2
  
  uint8_t i;
  for (i = 0; (i < len) && Wire.available(); i++)
  
  {
    pBuf[i] = Wire.read();
  }
  
  return i;
}

uint8_t readgt911TouchLocation( TouchLocation * pLoc, uint8_t num )
{
  uint8_t retVal;
  uint8_t i;
  uint8_t k;
  uint8_t  ss[1];
  
  do
  {  
    
    if (!pLoc) break; // must have a buffer
    if (!num)  break; // must be able to take at least one
     ss[0]=0;
      readgt911TouchAddr( 0x814e, ss, 1);
      uint8_t status=ss[0];

    if ((status & 0x0f) == 0) break; // no points detected
    uint8_t hitPoints = status & 0x0f;
    
    Serial.print("number of hit points = ");
    Serial.println( hitPoints );
    
   uint8_t tbuf[32]; uint8_t tbuf1[32];uint8_t tbuf2[16];  
    readgt911TouchAddr( 0x8150, tbuf, 32);
    readgt911TouchAddr( 0x8150+32, tbuf1, 32);
    
      if(hitPoints<=4)
            {   
              for (k=0,i = 0; (i <  4*8)&&(k < num); k++, i += 8)
              {
                pLoc[k].x = tbuf[i+1] << 8 | tbuf[i+0];
                pLoc[k].y = tbuf[i+3] << 8 | tbuf[i+2];
              }   
            }
        if(hitPoints>4)   
            {  
               for (k=0,i = 0; (i <  4*8)&&(k < num); k++, i += 8)
              {
                pLoc[k].x = tbuf[i+1] << 8 | tbuf[i+0];
                pLoc[k].y = tbuf[i+3] << 8 | tbuf[i+2];
              }               
              
              for (k=4,i = 0; (i <  4*8)&&(k < num); k++, i += 8)
              {
                pLoc[k].x = tbuf1[i+1] << 8 | tbuf1[i+0];
                pLoc[k].y = tbuf1[i+3] << 8 | tbuf1[i+2];
              }   
            } 
            
                
    
    retVal = hitPoints;
    
  } while (0);
  
    ss[0]=0;
    writegt911TouchRegister( 0x814e,ss,1); 
    delay(2);  
  return retVal;
}


void inttostr(uint16_t value,uint8_t *str)
{
	str[0]=value/1000+0x30;
	str[1]=value%1000/100+0x30;
	str[2]=value%1000%100/10+0x30;
	str[3]=value%1000%100%10+0x30;

}


void setup() 
{
  Serial.begin(9600);
  Serial.println("RA8875 start");
  Wire.begin();        // join i2c bus (address optional for master)


     pinMode(RA8875_RESET, OUTPUT); 
    pinMode     (gt911_INT, OUTPUT);
    digitalWrite(RA8875_RESET, LOW);
    digitalWrite(gt911_INT, LOW);
    delay(20);
     digitalWrite(RA8875_RESET, HIGH);
     delay(50);  
    pinMode     (gt911_INT, INPUT);
     delay(100);  
   
    uint8_t re=gt911_Send_Cfg((uint8_t*)GTP_CFG_DATA,sizeof(GTP_CFG_DATA));
    


         uint8_t bb[2];
 	 readgt911TouchAddr(0x8047,bb,2);  
         while(bb[1]!=32)
         {Serial.println("Capacitive touch screen initialized failure"); 
        pinMode(RA8875_RESET, OUTPUT); 
    pinMode     (gt911_INT, OUTPUT);
    digitalWrite(RA8875_RESET, LOW);
        delay(20);
    digitalWrite(gt911_INT, LOW);
    delay(50);
     digitalWrite(RA8875_RESET, HIGH);
     delay(100);  
    pinMode     (gt911_INT, INPUT);
     delay(100);      
      re=gt911_Send_Cfg((uint8_t*)GTP_CFG_DATA,sizeof(GTP_CFG_DATA));
       }



  while (!tft.begin(RA8875_800x480)) 
  {
    Serial.println("RA8875 Not Found!");
    delay(100);
  }
  Serial.println("Found RA8875");
  

  tft.displayOn(true);
  tft.GPIOX(true);      // Enable TFT - display enable tied to GPIOX
  tft.PWM1config(true, RA8875_PWM_CLK_DIV1024); // PWM output for backlight
  tft.PWM1out(255);

/*
  // With hardware accelleration this is instant
  tft.fillScreen(RA8875_WHITE);

      tft.textColor(RA8875_RED, RA8875_BLACK);

                  uint8_t bb[32];
	 	readgt911TouchAddr(0x8047,bb,32);

		  
		uint8_t in=0; 
                uint8_t ss[4];
              tft.textMode();
              tft.textSetCursor(0,50);
	 while(in<32)				
		{inttostr(bb[in],ss);
                tft.writeCommand(0x02);
		 tft.writeData(ss[0]);	    delay(1);   	
		 tft.writeData(ss[1]);	  delay(1);   
		 tft.writeData(ss[2]);	  delay(1);   
		 tft.writeData(ss[3]);	delay(1);   
		tft.writeData(',');	        delay(1);   
		in+=1;
		}

    delay(3000);  


*/



      tft.textMode();
      tft.textSetCursor(0,0);
      tft.textColor(RA8875_RED, RA8875_BLACK);
      tft.textWrite("www.buydiplay.com  Backlight brightness adjustment test ");
      tft.graphicsMode();

  // Play with PWM
  for (uint8_t i=255; i!=0; i-=5 ) 
  {
    tft.PWM1out(i); 
    delay(10);
  }  
  for (uint8_t i=0; i!=255; i+=5 ) 
  {
    tft.PWM1out(i); 
    delay(10);
  }
  tft.PWM1out(255); 
  
  tft.fillScreen(RA8875_RED);
  delay(500);
  tft.fillScreen(RA8875_YELLOW);
  delay(500);
  tft.fillScreen(RA8875_GREEN);
  delay(500);
  tft.fillScreen(RA8875_CYAN);
  delay(500);
  tft.fillScreen(RA8875_MAGENTA);
  delay(500);
  tft.fillScreen(RA8875_BLACK);
  
  // Try some GFX acceleration!
  tft.drawCircle(100, 100, 50, RA8875_BLACK);
  tft.fillCircle(100, 100, 49, RA8875_GREEN);
  
  tft.fillRect(11, 11, 700, 400, RA8875_BLUE);
  tft.drawRect(10, 10, 400, 200, RA8875_GREEN);
  tft.fillRoundRect(200, 10, 200, 100, 10, RA8875_RED);
  tft.fillRoundRect(480, 100, 300, 200, 10, RA8875_RED);
  tft.drawPixel(10,10,RA8875_BLACK);
  tft.drawPixel(11,11,RA8875_BLACK);
  //tft.drawLine(10, 10, 200, 100, RA8875_RED);
  tft.drawTriangle(200, 15, 250, 100, 150, 125, RA8875_BLACK);
  //tft.fillTriangle(200, 16, 249, 99, 151, 460, RA8875_YELLOW);
  tft.drawEllipse(300, 100, 100, 40, RA8875_BLACK);
  tft.fillEllipse(300, 100, 98, 38, RA8875_GREEN);
  // Argument 5 (curvePart) is a 2-bit value to control each corner (select 0, 1, 2, or 3)
  tft.drawCurve(50, 100, 80, 40, 2, RA8875_BLACK);  
  tft.fillCurve(50, 100, 78, 38, 2, RA8875_WHITE);
  
  
  tft.touchEnable(false);
    
  Serial.print("Status: "); Serial.println(tft.readStatus(), HEX);
  Serial.println("Waiting for touch events ...");
  
  randomSeed(analogRead(0));
}

uint32_t dist(const TouchLocation & loc)
{
  uint32_t retVal = 0;
  
  uint32_t x = loc.x;
  uint32_t y = loc.y;
  
  retVal = x*x + y*y;
  
  return retVal;
}

uint32_t dist(const TouchLocation & loc1, const TouchLocation & loc2)
{
  uint32_t retVal = 0;
  
  uint32_t x = loc1.x - loc2.x;
  uint32_t y = loc1.y - loc2.y;
  
  retVal = sqrt(x*x + y*y);
  
  return retVal;
}

bool sameLoc( const TouchLocation & loc, const TouchLocation & loc2 )
{
  return dist(loc,loc2) < 50;
}

void loop() 
{
  static uint16_t total = 256;
  static uint16_t w = tft.width();
  static uint16_t h = tft.height();
  
  float xScale = 1024.0F/w;
  float yScale = 1024.0F/h;
 


  uint8_t i=0;   
  

    while(total!=0)
  {
    total--;
    if ((total & 0x3ff) == 0)
    {
      //Serial.println(total);
    }
    // void    fillRect(int16_t x, int16_t y, int16_t w, int16_t h, uint16_t color);
    int16_t x  = random(w);
    int16_t y  = random(h);
    int16_t x1 = random(w);
    int16_t y1 = random(h);
    int16_t x2 = random(w);
    int16_t y2 = random(h);
    uint16_t c  = (((random(64)<<6)|random(64))<<6)|random(31);
    uint16_t bg = (((random(64)<<6)|random(64))<<6)|random(31);
    int16_t th = random(h);
    int16_t tw = random(w);
    int16_t op = random(9);
    int16_t en = random(8);
    
    switch( op )
    {
    default:
    case 0:
      tft.fillRect(x,y, tw, th, c);
      break;
    case 1:
      tft.fillEllipse(x,y,tw/2,th/2,c);
      break;
    case 2:
      tft.fillCircle(x,y,tw/2,c);
      break;
    case 3:
      tft.fillTriangle(x,y,x1,y1,x2,y2,c);
      break;
    case 4:
      tft.textMode();
      tft.textSetCursor(x,y);
      tft.textColor(c,bg);
      tft.textEnlarge(en);
      tft.textWrite("Hallowell",9);
      tft.graphicsMode();
      break;
    case 5:
      tft.drawRect(x,y, tw, th, c);
      break;
    case 6:
      tft.drawEllipse(x,y,tw/2,th/2,c);
      break;
    case 7:
      tft.drawCircle(x,y,tw/2,c);
      break;
    case 8:
      tft.drawTriangle(x,y,x1,y1,x2,y2,c);
      break;
    }
     if(total==0)
       {  tft.fillScreen(RA8875_BLACK);
         tft.textMode();
        tft.textSetCursor(0,0);
        tft.textColor(RA8875_RED, RA8875_BLACK);
         tft.textEnlarge(0);
        tft.textWrite("www.buydiplay.com  Capacitive touch screen test.Please touch the screen!");
              tft.textSetCursor(w-40,0); 
         tft.textWrite("CLEAR");     
       }      
  }
  
  /* Wait around for touch events */
  
        pinMode     (gt911_INT, INPUT);  
       uint8_t st=digitalRead(gt911_INT); 
    if(!st)   //Hardware touch interrupt
    
       /*
       uint8_t  ss[1];
      readgt911TouchAddr( 0x814e, ss, 1);
      uint8_t status=ss[0];
    if ((status & 0x80) != 0)  // touch status   Software touch interrupt  
    */
    {   
      Serial.println("Touch: ");
      
      uint8_t count = readgt911TouchLocation( touchLocations, 5 );
      
      //static uint8_t lastCount = count;
  
     if (count)
      {

        
        for (i = 0; i < count; i++)
        {
    
          tft.textMode();
        tft.textSetCursor(0,0);
        tft.textColor(RA8875_RED, RA8875_BLACK);
         tft.textEnlarge(0);
      tft.textWrite("www.buydiplay.com  Capacitive touch screen test.Please touch the screen!");
          //tft.textSetCursor(touchLocations[i].x,touchLocations[i].y);
          tft.textSetCursor(380,200+16*i);
          tft.textColor(RA8875_WHITE, RA8875_BLACK);
          tft.textEnlarge(0);
          snprintf((char*)buf,sizeof(buf),"(%3d,%3d)",touchLocations[i].x,touchLocations[i].y);
          tft.textWrite((const char *)buf,9);
        tft.graphicsMode();  
        if(i==0)  tft.fillCircle(touchLocations[i].x,touchLocations[i].y, 3, RA8875_RED);  
        else if(i==1)  tft.fillCircle(touchLocations[i].x,touchLocations[i].y, 3, RA8875_GREEN); 
        else if(i==2)  tft.fillCircle(touchLocations[i].x,touchLocations[i].y, 3, RA8875_BLUE);        
        else if(i==3)  tft.fillCircle(touchLocations[i].x,touchLocations[i].y, 3, RA8875_CYAN); 
         else if(i==4)  tft.fillCircle(touchLocations[i].x,touchLocations[i].y, 3, RA8875_MAGENTA);  
         tft.textMode();     
        }
      }
        
          if(touchLocations[0].x>=760&&(touchLocations[0].y&0x0fff)<=30)
          {tft.fillScreen(RA8875_BLACK);
           tft.textMode();
          tft.textSetCursor(0,0);
          tft.textColor(RA8875_RED, RA8875_BLACK);
          tft.textEnlarge(0);
          tft.textWrite("www.buydiplay.com  Capacitive touch screen test.Please touch the screen!");  
           tft.textSetCursor(w-40,0); 
           tft.textWrite("CLEAR");  
          } 
    }
  
    else
    {
    }
    
   
}
